/*=========================================================================

  Library   : Image Registration Toolkit ()
  Module    : $Id$
  Copyright : Imperial College, Department of Computing
              Visual Information Processing (VIP), 2008 onwards
  Date      : $Date$
  Version   : $Revision$
  Changes   : $Author$

=========================================================================*/

#ifndef _COMPARISONTABLE_H
#define _COMPARISONTABLE_H

#include <limits>
#include <mirtk/Segment.h>


class ComparisonTable
{

  friend class RView;
  friend class Viewer;

protected:

  /// Segment Table
  Segment _centry[std::numeric_limits<short>::max() + 2]; //originally +1

 int _ccColor[3];
 int _diff1Color[3];
 int _diff2Color[3];
 int _seg1Color[3];
 int _seg2Color[3];
 int  _intersectionColor[3];

 int _ccColorC[3];
 int _diff1ColorC[3];
 int _diff2ColorC[3];
 int _seg1ColorC[3];
 int _seg2ColorC[3];
 int  _intersectionColorC[3];



 public:

/// Constructor (basic)
  ComparisonTable();

  /// Constructor (default)
  ComparisonTable(int);

  /// Destructor
  virtual ~ComparisonTable();

  /// Size of lookup table
  int Size();

  /// Sets all values for a segment
  void Set(int, char*, unsigned char, unsigned char, unsigned char, double, int);

  /// Sets the label value of sid (index)
  void SetLabel(int, char*);

  /// Sets the value of r,g,b as unsigned char of sid (index)
  void SetColor(int, unsigned char, unsigned char, unsigned char);

  ///Sets second segmentation color
  void SetSecondColor(int, unsigned char, unsigned char, unsigned char);

  /// Sets the transparency of sid (index)
  void SetTrans(int, double);

  /// Sets the visibility
  void SetVisibility(int, int);

  /// Get values at index id (int)
  char *Get(int, unsigned char*, unsigned char*, unsigned char*, double*, int*) const;

  /// Get segment colours at index id (int)
  void GetColor(int, unsigned char*, unsigned char*, unsigned char*);

  int  Get_ColorR(char);
  int  Get_ColorG(char);
  int  Get_ColorB(char);

  void  Set_Color(char,int*);




























  /// Get segment transparency value (double)
  void GetTrans(int, double*);

  /// Get segment hex value (char *)
  void GetHex(int, char*);

  /// Get segment label (char *)
  char *GetLabel(int);

  /// Is method that finds whether a segment is visible
  int GetVisibility(int);

  /// Find if entry contains valid value
  int IsValid(int);

  /// Remove segment with index
  void Clear(int);

  /// Remove all segments
  void Clear();

  /// Reads a segmentTable from a file
  void Read(char *);

  /// Writes a segmentTable to a file
  void Write(char *);




  void SetColor(int);

  void GetColor (int);






};

inline void ComparisonTable::GetColor(int id, unsigned char* r, unsigned char* g, unsigned char* b)
{
  _centry[id].getColor(r, g, b);
}

inline void ComparisonTable::GetTrans(int id, double* d)
{
  *d = _centry[id].getTrans();
}

inline void ComparisonTable::GetHex(int id, char* h)
{
	_centry[id].getHex(h);
}

inline char *ComparisonTable::GetLabel(int id)
{
  return _centry[id].getLabel();
}

inline int ComparisonTable::GetVisibility(int id)
{
  return _centry[id].getVisibility();
}

inline int ComparisonTable::IsValid(int id)
{
  if (_centry[id].getLabel() != NULL) {

    return true;
  } else {
    return false;
  }
}

inline int ComparisonTable::Size()
{
  return std::numeric_limits<short>::max() + 1;
}

#endif

