/*=========================================================================

  Library   : Image Registration Toolkit (IRTK)
  Module    : $Id$
  Copyright : Imperial College, Department of Computing
              Visual Information Processing (VIP), 2008 onwards
  Date      : $Date$
  Version   : $Revision$
  Changes   : $Author$

=========================================================================*/

#include <mirtk/Image.h>
#include <mirtk/Transformation.h>
#include <mirtk/Registration.h>

#include <mirtk/Segment.h>
#include <FL/Fl_Window.H>

// Local includes
#include "Fl_RViewUI.h"

#ifdef HAS_COMPARISON_PANEL

#include "bitmaps/kchart.xpm"
#include "bitmaps/color_line_closed.xpm"
#include "bitmaps/color_line_open.xpm"
#include "bitmaps/fileopen.xpm"

extern Fl_RViewUI  *rviewUI;
extern Fl_RView    *viewer;
extern RView    *rview;
Segment *interseclabel;



//char widthStrings[10][10] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"};



//char growStrings[2][10] = {"2D", "3D"};



void Fl_RViewUI::cb_loadLabel1(Fl_Button *, void *)
{
  char *filename = fl_file_chooser("Load segmentation image", "*.{gipl,gipl.gz,hdr,hdr.gz,nii,nii.gz,vtk}", "");
  if (filename != NULL) {
    rview->ReadLabel1(filename);
    //rviewUI->_id = -1;
    //rviewUI->_selected = 0;
    if (rviewUI->_histogramWindow!=NULL) rviewUI->_histogramWindow->recalculate();

    // Update
    rview->ComparisonUpdateOn();
    rview->Update();
    viewer->redraw();
    rviewUI->update();
  }
}
void Fl_RViewUI::cb_loadLabel2(Fl_Button *, void *)
{
  char *filename = fl_file_chooser("Load segmentation image", "*.{gipl,gipl.gz,hdr,hdr.gz,nii,nii.gz,vtk}", "");
  if (filename != NULL) {
    rview->ReadLabel2(filename);
    //rviewUI->_id = -1;
    //rviewUI->_selected = 0;
    if (rviewUI->_histogramWindow!=NULL) rviewUI->_histogramWindow->recalculate();

    // Update
    rview->ComparisonUpdateOn();
    rview->Update();
    viewer->redraw();
    rviewUI->update();
  }
}




















void Fl_RViewUI::cb_display_DoubleContour(Fl_Check_Button*o, void*)
{
  if (o->value() == 1) {
    rview->DoubleContourOn();
  } else {
    rview->DoubleContourOff();
  }
  // Update
  rview->ComparisonUpdateOn();
  rview->Update();
  rviewUI->update();
  viewer->redraw();
}



void Fl_RViewUI::cb_displayComparisonContours(Fl_Check_Button *o, void *)
{
  if (o->value() == 1) {
    rview->ComparisonContoursOn();
  } else {
    rview->ComparisonContoursOff();
  }
  // Update
  rview->ComparisonUpdateOn();
  rview->Update();
  rviewUI->update();
  viewer->redraw();
}
void Fl_RViewUI::cb_displayIntersection(Fl_Check_Button *o, void *)
{
  if (o->value() == 1) {
    rview->IntersectionOn();
  } else {
    rview->IntersectionOff();
  }
  // Update
  rview->ComparisonUpdateOn();
  rview->Update();
  rviewUI->update();
  viewer->redraw();
}
void Fl_RViewUI::cb_displayDiff1(Fl_Check_Button *o, void *)
{
  if (o->value() == 1) {
    rview->Diff1On();
  } else {
    rview->Diff1Off();
  }
  // Update
  rview->ComparisonUpdateOn();
  rview->Update();
  rviewUI->update();
  viewer->redraw();
}
void Fl_RViewUI::cb_displayDiff2(Fl_Check_Button *o, void *)
{
  if (o->value() == 1) {
    rview->Diff2On();
  } else {
    rview->Diff2Off();
  }
  // Update
  rview->ComparisonUpdateOn();
  rview->Update();
  rviewUI->update();
  viewer->redraw();
}



void Fl_RViewUI::cb_displayImage1(Fl_Check_Button *o, void *)
{
  if (o->value() == 1) {
    rview->Image1On();
  } else {
    rview->Image1Off();
  }
  // Update
  rview->ComparisonUpdateOn();
  rview->Update();
  rviewUI->update();
  viewer->redraw();
}

void Fl_RViewUI::cb_displayImage2(Fl_Check_Button *o, void *)
{
  if (o->value() == 1) {
    rview->Image2On();
  } else {
    rview->Image2Off();
  }
  // Update
  rview->ComparisonUpdateOn();
  rview->Update();
  rviewUI->update();
  viewer->redraw();
}

void Fl_RViewUI::cb_displayCC(Fl_Check_Button *o, void *)
{
  if (o->value() == 1) {
    rview->CCOn();
  } else {
    rview->CCOff();
  }
  // Update
  rview->ComparisonUpdateOn();
  rview->Update();
  rviewUI->update();
  viewer->redraw();
}








 void Fl_RViewUI::cb_changeColorCC(Fl_Button*, void*){


	   double r = rviewUI->_redC / 255.0;
	   double g = rviewUI->_greenC / 255.0;
	   double b = rviewUI->_blueC / 255.0;
	   int color = fl_color_chooser("Color Selection", r, g, b);

	   int c[3];

	   if (color > 0) {
	     rviewUI->_redC   = round(r * 255);
	     rviewUI->_greenC = round(g * 255);
	     rviewUI->_blueC = round(b * 255);

	     c[0]= rviewUI->_redC ;
	 	c[1]= rviewUI->_greenC;
	     c[2]= rviewUI->_blueC ;
	   }

	   rviewUI->changeColorCC->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
	   // Update color
	   rview->GetComparisonTable()->Set_Color('a',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

	   // Update
	   rview->ComparisonUpdateOn();
	   rview->Update();
	   rviewUI->update();
	   viewer->redraw();

	  }
  void Fl_RViewUI::cb_changeColorDiff1(Fl_Button*, void*){


	   double r = rviewUI->_redC / 255.0;
	   double g = rviewUI->_greenC / 255.0;
	   double b = rviewUI->_blueC / 255.0;
	   int color = fl_color_chooser("Color Selection", r, g, b);

	   int c[3];

	   if (color > 0) {
	     rviewUI->_redC   = round(r * 255);
	     rviewUI->_greenC = round(g * 255);
	     rviewUI->_blueC  = round(b * 255);

	     c[0]= rviewUI->_redC ;
	 	c[1]= rviewUI->_greenC;
	     c[2]= rviewUI->_blueC ;
	   }

	   // Update color
	   rviewUI->changeColorDiff1->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
	   rview->GetComparisonTable()->Set_Color('b',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

	   // Update
	   rview->ComparisonUpdateOn();
	   rview->Update();
	   rviewUI->update();
	   viewer->redraw();

	  }
  void Fl_RViewUI::cb_changeColorDiff2(Fl_Button*, void*){


	   double r = rviewUI->_redC / 255.0;
	   double g = rviewUI->_greenC / 255.0;
	   double b = rviewUI->_blueC / 255.0;
	   int color = fl_color_chooser("Color Selection", r, g, b);

	   int c[3];

	   if (color > 0) {
	     rviewUI->_redC   = round(r * 255);
	     rviewUI->_greenC = round(g * 255);
	     rviewUI->_blueC  = round(b * 255);

	     c[0]= rviewUI->_redC ;
	 	c[1]= rviewUI->_greenC;
	     c[2]= rviewUI->_blueC ;
	   }

	   // Update color
	   rviewUI->changeColorDiff2->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
	   rview->GetComparisonTable()->Set_Color('c',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

	   // Update
	   rview->ComparisonUpdateOn();
	   rview->Update();
	   rviewUI->update();
	   viewer->redraw();

	  }
  void Fl_RViewUI::cb_changeColorImg1(Fl_Button*, void*){


	   double r = rviewUI->_redC / 255.0;
	   double g = rviewUI->_greenC / 255.0;
	   double b = rviewUI->_blueC / 255.0;
	   int color = fl_color_chooser("Color Selection", r, g, b);

	   int c[3];

	   if (color > 0) {
	     rviewUI->_redC   = round(r * 255);
	     rviewUI->_greenC = round(g * 255);
	     rviewUI->_blueC  = round(b * 255);

	     c[0]= rviewUI->_redC ;
	 	c[1]= rviewUI->_greenC;
	     c[2]= rviewUI->_blueC ;
	   }

	   // Update color
	   rviewUI->changeColorImg1->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
	   rview->GetComparisonTable()->Set_Color('d',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

	   // Update
	   rview->ComparisonUpdateOn();
	   rview->Update();
	   rviewUI->update();
	   viewer->redraw();

	  }
  void Fl_RViewUI::cb_changeColorImg2(Fl_Button*, void*){


	   double r = rviewUI->_redC / 255.0;
	   double g = rviewUI->_greenC / 255.0;
	   double b = rviewUI->_blueC / 255.0;
	   int color = fl_color_chooser("Color Selection", r, g, b);

	   int c[3];

	   if (color > 0) {
	     rviewUI->_redC   = round(r * 255);
	     rviewUI->_greenC = round(g * 255);
	     rviewUI->_blueC  = round(b * 255);

	     c[0]= rviewUI->_redC ;
	 	c[1]= rviewUI->_greenC;
	     c[2]= rviewUI->_blueC ;
	   }

	   // Update color
	   rviewUI->changeColorImg2->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
	   rview->GetComparisonTable()->Set_Color('e',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

	   // Update
	   rview->ComparisonUpdateOn();
	   rview->Update();
	   rviewUI->update();
	   viewer->redraw();

	  }
  void Fl_RViewUI::cb_changeColorIntersec(Fl_Button*, void*){


   double r = rviewUI->_redC / 255.0;
   double g = rviewUI->_greenC / 255.0;
   double b = rviewUI->_blueC / 255.0;
   int color = fl_color_chooser("Color Selection", r, g, b);

   int c[3];

   if (color > 0) {
     rviewUI->_redC   = round(r * 255);
     rviewUI->_greenC = round(g * 255);
     rviewUI->_blueC  = round(b * 255);

     c[0]= rviewUI->_redC ;
 	c[1]= rviewUI->_greenC;
     c[2]= rviewUI->_blueC ;
   }

   // Update color
   rviewUI->changeColorIntersec->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
   rview->GetComparisonTable()->Set_Color('f',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

   // Update
   rview->ComparisonUpdateOn();
   rview->Update();
   rviewUI->update();
   viewer->redraw();

  }



  void Fl_RViewUI::cb_changeColorCCC(Fl_Button*, void*){


  	   double r = rviewUI->_redC / 255.0;
  	   double g = rviewUI->_greenC / 255.0;
  	   double b = rviewUI->_blueC / 255.0;
  	   int color = fl_color_chooser("Color Selection", r, g, b);

  	   int c[3];

  	   if (color > 0) {
  	     rviewUI->_redC   = round(r * 255);
  	     rviewUI->_greenC = round(g * 255);
  	     rviewUI->_blueC  = round(b * 255);

  	     c[0]= rviewUI->_redC ;
  	 	c[1]= rviewUI->_greenC;
  	     c[2]= rviewUI->_blueC ;
  	   }

  	   rviewUI->changeColorCCC->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
  	   // Update color
  	   rview->GetComparisonTable()->Set_Color('g',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

  	   // Update
  	   rview->ComparisonUpdateOn();
  	   rview->Update();
  	   rviewUI->update();
  	   viewer->redraw();

  	  }
    void Fl_RViewUI::cb_changeColorDiff1C(Fl_Button*, void*){


  	   double r = rviewUI->_redC / 255.0;
  	   double g = rviewUI->_greenC / 255.0;
  	   double b = rviewUI->_blueC / 255.0;
  	   int color = fl_color_chooser("Color Selection", r, g, b);

  	   int c[3];

  	   if (color > 0) {
  	     rviewUI->_redC   = round(r * 255);
  	     rviewUI->_greenC = round(g * 255);
  	     rviewUI->_blueC  = round(b * 255);

  	     c[0]= rviewUI->_redC ;
  	 	c[1]= rviewUI->_greenC;
  	     c[2]= rviewUI->_blueC ;
  	   }

  	   // Update color
  	   rviewUI->changeColorDiff1C->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
  	   rview->GetComparisonTable()->Set_Color('h',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

  	   // Update
  	   rview->ComparisonUpdateOn();
  	   rview->Update();
  	   rviewUI->update();
  	   viewer->redraw();

  	  }
    void Fl_RViewUI::cb_changeColorDiff2C(Fl_Button*, void*){


  	   double r = rviewUI->_redC / 255.0;
  	   double g = rviewUI->_greenC / 255.0;
  	   double b = rviewUI->_blueC / 255.0;
  	   int color = fl_color_chooser("Color Selection", r, g, b);

  	   int c[3];

  	   if (color > 0) {
  	     rviewUI->_redC   = round(r * 255);
  	     rviewUI->_greenC = round(g * 255);
  	     rviewUI->_blueC  = round(b * 255);

  	     c[0]= rviewUI->_redC ;
  	 	c[1]= rviewUI->_greenC;
  	     c[2]= rviewUI->_blueC ;
  	   }

  	   // Update color
  	   rviewUI->changeColorDiff2C->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
  	   rview->GetComparisonTable()->Set_Color('i',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

  	   // Update
  	   rview->ComparisonUpdateOn();
  	   rview->Update();
  	   rviewUI->update();
  	   viewer->redraw();

  	  }
    void Fl_RViewUI::cb_changeColorImg1C(Fl_Button*, void*){


  	   double r = rviewUI->_redC / 255.0;
  	   double g = rviewUI->_greenC / 255.0;
  	   double b = rviewUI->_blueC / 255.0;
  	   int color = fl_color_chooser("Color Selection", r, g, b);

  	   int c[3];

  	   if (color > 0) {
  	     rviewUI->_redC   = round(r * 255);
  	     rviewUI->_greenC = round(g * 255);
  	     rviewUI->_blueC  = round(b * 255);

  	     c[0]= rviewUI->_redC ;
  	 	c[1]= rviewUI->_greenC;
  	     c[2]= rviewUI->_blueC ;
  	   }

  	   // Update color
  	   rviewUI->changeColorImg1C->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
  	   rview->GetComparisonTable()->Set_Color('j',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

  	   // Update
  	   rview->ComparisonUpdateOn();
  	   rview->Update();
  	   rviewUI->update();
  	   viewer->redraw();

  	  }
    void Fl_RViewUI::cb_changeColorImg2C(Fl_Button*, void*){


  	   double r = rviewUI->_redC / 255.0;
  	   double g = rviewUI->_greenC / 255.0;
  	   double b = rviewUI->_blueC / 255.0;
  	   int color = fl_color_chooser("Color Selection", r, g, b);

  	   int c[3];

  	   if (color > 0) {
  	     rviewUI->_redC   = round(r * 255);
  	     rviewUI->_greenC = round(g * 255);
  	     rviewUI->_blueC  = round(b * 255);

  	     c[0]= rviewUI->_redC ;
  	 	c[1]= rviewUI->_greenC;
  	     c[2]= rviewUI->_blueC ;
  	   }

  	   // Update color
  	   rviewUI->changeColorImg2C->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
  	   rview->GetComparisonTable()->Set_Color('k',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

  	   // Update
  	   rview->ComparisonUpdateOn();
  	   rview->Update();
  	   rviewUI->update();
  	   viewer->redraw();

  	  }
    void Fl_RViewUI::cb_changeColorIntersecC(Fl_Button*, void*){


     double r = rviewUI->_redC / 255.0;
     double g = rviewUI->_greenC / 255.0;
     double b = rviewUI->_blueC / 255.0;
     int color = fl_color_chooser("Color Selection", r, g, b);

     int c[3];

     if (color > 0) {
       rviewUI->_redC   = round(r * 255);
       rviewUI->_greenC = round(g * 255);
       rviewUI->_blueC  = round(b * 255);

       c[0]= rviewUI->_redC ;
   	   c[1]= rviewUI->_greenC;
       c[2]= rviewUI->_blueC ;
     }

     // Update color
     rviewUI->changeColorIntersecC->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
     rview->GetComparisonTable()->Set_Color('l',c);//->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));

     // Update
     rview->ComparisonUpdateOn();
     rview->Update();
     rviewUI->update();
     viewer->redraw();

    }











void Fl_RViewUI::ShowComparisonControlWindow()
{
}

void Fl_RViewUI::UpdateComparisonBrowser()
{
 // int i;

  // Clear browser
 // rviewUI->segmentObjectBrowser->clear();

  // Add labels
 // for (i = 0; i <= rview->GetSegmentTable()->Size(); i++) {
 //   if (rview->GetSegmentTable()->IsValid(i) == true) {
 //     char buffer[256];
 //     sprintf(buffer, "%d \t %s", i, rview->GetSegmentTable()->GetLabel(i));
  //    rviewUI->segmentObjectBrowser->add(buffer);
 //   }
 // }
}

void Fl_RViewUI::UpdateComparisonControlWindow()
{
/*

  if (rviewUI->_id == -1) {
    rviewUI->editSegmentColor->color(fl_rgb_color(255, 255, 255));
    rviewUI->editSegmentColor->deactivate();
    rviewUI->editSegmentTransperancy->value(1);
    rviewUI->editSegmentTransperancy->deactivate();
    rviewUI->editSegmentVisibility->value(1);
    rviewUI->editSegmentVisibility->deactivate();
    rviewUI->editSegmentLabel->value("");
    rviewUI->editSegmentLabel->deactivate();
  } else {
    rviewUI->editSegmentColor->color(fl_rgb_color(rviewUI->_redC, rviewUI->_greenC, rviewUI->_blueC));
    rviewUI->editSegmentColor->activate();
    rviewUI->editSegmentTransperancy->value(rviewUI->_trans);
    rviewUI->editSegmentTransperancy->activate();
    rviewUI->editSegmentVisibility->value(rviewUI->_vis);
    rviewUI->editSegmentVisibility->activate();
    rviewUI->editSegmentLabel->value(rviewUI->_label);
    rviewUI->editSegmentLabel->activate();
  }
  rviewUI->editSegmentColor->redraw();

  // Update viewing mode
  rviewUI->drawContour->value(0);
  rviewUI->drawPaintBrush->value(0);
  switch (rview->GetSegmentationMode()) {
  case 0:
    rviewUI->drawPaintBrush->value(1);
    break;
  case 1:
    rviewUI->drawContour->value(1);
    break;
  }




  switch (rview->GetPaintBrushWidth()) {
  case 1:
    rviewUI->paintBrushWidth->value(0);
    break;
  case 2:
    rviewUI->paintBrushWidth->value(1);
    break;
  case 3:
    rviewUI->paintBrushWidth->value(2);
    break;
  case 4:
    rviewUI->paintBrushWidth->value(3);
    break;
  case 5:
    rviewUI->paintBrushWidth->value(4);
    break;
  case 6:
    rviewUI->paintBrushWidth->value(5);
    break;
  case 7:
    rviewUI->paintBrushWidth->value(6);
    break;
  case 8:
    rviewUI->paintBrushWidth->value(7);
    break;
  case 9:
    rviewUI->paintBrushWidth->value(8);
    break;
  }

  if ((rviewUI->_histogramWindow != NULL) && (rviewUI->_histogramWindow->shown())) rviewUI->_histogramWindow->draw();
*/}

void Fl_RViewUI::InitializeComparisonControlWindow()

{
 // _id = -1;
 // _selected=0;
  {



    Fl_Group* o1 = new Fl_Group(0, 0, 400, 500, "NewSegmentBrowserWindow");
    	     o1->user_data((void*)(this));
    	     o1->box(FL_ENGRAVED_BOX);
    	     o1->labeltype(FL_NO_LABEL);
    	     {	//Buttons for Loading
    	    	 Fl_Button *oB =new Fl_Button(50, 40, 2, 2, "Load binary");
    	    	 Fl_Button *oC =new Fl_Button(150, 40, 2, 2, "Segmentations");
    	    	 Fl_Button* o1 = new Fl_Button(70, 50, 32, 32);
    	    	 Fl_Button* o2 = new Fl_Button(110, 50, 32, 32);
    	    	 //Colorbuttons
    	    	 Fl_Button *oA =new Fl_Button(185, 150, 2, 2, "Inside");
    	    	 Fl_Button *o3 =changeColorCC=  new Fl_Button(160, 150, 50, 30, "");
    	    	 Fl_Button *o4 = changeColorDiff1= new Fl_Button(160, 200, 50, 30, "");
    	    	 Fl_Button *o5 =changeColorDiff2= new Fl_Button(160, 250, 50, 30, "");
    	    	 Fl_Button *o6 =changeColorImg1=  new Fl_Button(160, 300, 50, 30, "");
    	    	 Fl_Button *o7 =changeColorImg2=  new Fl_Button(160, 350, 50, 30, "");
    	    	 Fl_Button *o15=changeColorIntersec  = new Fl_Button(160, 400, 50, 30, "");
    	    	 //Doubel contour color buttons
    	    	 Fl_Button *o17 = changeColorCCC= new Fl_Button(240, 150, 50, 30, "Outside");
    	    	 Fl_Button *o18 = changeColorDiff1C= new Fl_Button(240, 200, 50, 30, "");
    	    	 Fl_Button *o19 = changeColorDiff2C= new Fl_Button(240, 250, 50, 30, "");
    	    	 Fl_Button *o20 = changeColorImg1C= new Fl_Button(240, 300, 50, 30, "");
    	    	 Fl_Button *o21 = changeColorImg2C=  new Fl_Button(240, 350, 50, 30, "");
    	    	 Fl_Button *o22 = changeColorIntersecC  = new Fl_Button(240, 400, 50, 30, "");
    	    	 // Visibility buttons
    	    	 Fl_Check_Button *o8 =
    	    			 new Fl_Check_Button(330, 150, 30,30,"Visibility");
    	    	 Fl_Check_Button *o9 =
    	    			 new Fl_Check_Button(330, 200, 30, 30,"");
    	    	 Fl_Check_Button *o10 =
    	    			 new Fl_Check_Button(330, 250, 30, 30,"");
    	    	 Fl_Check_Button *o11 =
    	    			new Fl_Check_Button(330, 300, 30, 30,"");
    	    	 Fl_Check_Button *o12 =
    	    			 new Fl_Check_Button(330, 350, 30, 30,"");
    	    	 Fl_Check_Button *o16 =
    	    	     	 new Fl_Check_Button(330, 400, 30, 30,"");
    	   // 	 Fl_Label *o23 =
    	    //	     	 new Fl_Label(330, 400, 30, 30,"Inside");



    	    	 Fl_Pixmap *p = new Fl_Pixmap(fileopen_xpm);


    	    	    p->label(o1);
    	    	    p->label(o2);
    	    	    o1->tooltip("Load segmentation 1");
    	    	    o2->tooltip("Load segmentation 2");
    	    	    o3->type(FL_NORMAL_BUTTON);
    	    	    o4->type(FL_NORMAL_BUTTON);
    	    	    o5->type(FL_NORMAL_BUTTON);
    	    	    o6->type(FL_NORMAL_BUTTON);
    	    	    o7->type(FL_NORMAL_BUTTON);
    	    	    o15->type(FL_NORMAL_BUTTON);
    	    	    o3->box(FL_BORDER_BOX);
    	    	    o4->box(FL_BORDER_BOX);
    	    	    o5->box(FL_BORDER_BOX);
    	    	    o6->box(FL_BORDER_BOX);
    	    	    o7->box(FL_BORDER_BOX);
    	    	    o15->box(FL_BORDER_BOX);
    	    	    o3->label("Contour Coincidence ");
    	    	    o4->label("1 \\ 2  ");
    	    	    o5->label("2 \\ 1 ");
    	    	    o6->label("Segmentation 1 ");
    	    	    o7->label("Segmentation 2 ");
    	    	    o15->label("Intersection");
    	    	    o3->align(FL_ALIGN_LEFT);
    	    	    o4->align(FL_ALIGN_LEFT);
    	    	    o5->align(FL_ALIGN_LEFT);
    	    	    o6->align(FL_ALIGN_LEFT);
    	    	    o7->align(FL_ALIGN_LEFT);
    	    	    o8->align(FL_ALIGN_TOP);
    	    	    oA->align(FL_ALIGN_TOP);
    	    	    o15->align(FL_ALIGN_LEFT);
    	    	    o17->align(FL_ALIGN_TOP);
    	    	    o3->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('a'),rview->GetComparisonTable()->Get_ColorG('a'),rview->GetComparisonTable()->Get_ColorB('a')));
    	    	    o4->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('b'),rview->GetComparisonTable()->Get_ColorG('b'),rview->GetComparisonTable()->Get_ColorB('b')));
    	    	    o5->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('c'),rview->GetComparisonTable()->Get_ColorG('c'),rview->GetComparisonTable()->Get_ColorB('c')));
    	    	    o6->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('d'),rview->GetComparisonTable()->Get_ColorG('d'),rview->GetComparisonTable()->Get_ColorB('d')));
    	    	    o7->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('e'),rview->GetComparisonTable()->Get_ColorG('e'),rview->GetComparisonTable()->Get_ColorB('e')));
    	    	    o15->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('f'),rview->GetComparisonTable()->Get_ColorG('f'),rview->GetComparisonTable()->Get_ColorB('f')));
    	    	    o17->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('g'),rview->GetComparisonTable()->Get_ColorG('g'),rview->GetComparisonTable()->Get_ColorB('g')));
    	    	    o18->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('h'),rview->GetComparisonTable()->Get_ColorG('h'),rview->GetComparisonTable()->Get_ColorB('h')));
    	    	    o19->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('i'),rview->GetComparisonTable()->Get_ColorG('i'),rview->GetComparisonTable()->Get_ColorB('i')));
    	    	    o20->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('j'),rview->GetComparisonTable()->Get_ColorG('j'),rview->GetComparisonTable()->Get_ColorB('j')));
    	    	    o21->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('k'),rview->GetComparisonTable()->Get_ColorG('k'),rview->GetComparisonTable()->Get_ColorB('k')));
    	    	    o22->color(fl_rgb_color(rview->GetComparisonTable()->Get_ColorR('l'),rview->GetComparisonTable()->Get_ColorG('l'),rview->GetComparisonTable()->Get_ColorB('l')));

    	    	    o1->callback((Fl_Callback*)cb_loadLabel1);
    	    	    o2->callback((Fl_Callback*)cb_loadLabel2);
    	    	    o3->callback((Fl_Callback*)cb_changeColorCC);
    	    	    o4->callback((Fl_Callback*)cb_changeColorDiff1);
    	    	   o5->callback((Fl_Callback*)cb_changeColorDiff2);
    	    	    o6->callback((Fl_Callback*)cb_changeColorImg1);
    	    	    o7->callback((Fl_Callback*)cb_changeColorImg2);
    	    	   o15->callback((Fl_Callback*)cb_changeColorIntersec);

    	    	   o17->callback((Fl_Callback*)cb_changeColorCCC);
    	    	   o18->callback((Fl_Callback*)cb_changeColorDiff1C);
    	    	   o19->callback((Fl_Callback*)cb_changeColorDiff2C);
    	    	   o20->callback((Fl_Callback*)cb_changeColorImg1C);
    	    	   o21->callback((Fl_Callback*)cb_changeColorImg2C);
    	    	   o22->callback((Fl_Callback*)cb_changeColorIntersecC);






    	    	    o8->callback((Fl_Callback*) cb_displayCC);
    	    	    o9->callback((Fl_Callback*)cb_displayDiff1);
    	    	    o10->callback((Fl_Callback*) cb_displayDiff2);
    	    	    o11->callback((Fl_Callback*)cb_displayImage1);
    	    	    o12->callback((Fl_Callback*) cb_displayImage2);
    	    	    o16->callback((Fl_Callback*)cb_displayIntersection);




    	    	    Fl_Check_Button* o14 = new Fl_Check_Button(160, 440, 32, 32);
    	    	    o14->label("Double Contours ");
    	    	    o14->align(FL_ALIGN_LEFT);
    	    	    o14->callback((Fl_Callback*) cb_display_DoubleContour);





    	    	            //o->callback((Fl_Callback *)cb_editSegmentPickColor);



    	     }

    	   /*  Fl_Simple_Group *o2 = new Fl_Simple_Group(20, 40, 160, 60, "Load Binary Segmentations");
    	         	     	 	 	 o2->user_data((void*)(this));
    	         	         	     o2->box(FL_ENGRAVED_BOX);
    	         	         	     {
    	         	     	        {
    	         	     		  	  Fl_Button* o1 = new Fl_Button(40, 60, 32, 32);
    	         	     		  	  o1->box(FL_NO_BOX);
    	         	     		  	  o1->callback((Fl_Callback*)cb_loadLabel1);
    	         	     		  	  o1->tooltip("Load segmentation 1");
    	         	     	          Fl_Pixmap *p = new Fl_Pixmap(fileopen_xpm);
    	         	     	          p->label(o1);
    	         	     	        }
    	         	     	        {
    	         	     	          Fl_Button* o2 = new Fl_Button(110, 60, 32, 32);
    	         	     	          o2->box(FL_NO_BOX);
    	         	     	          o2->callback((Fl_Callback*)cb_loadLabel2);
    	         	     	          o2->tooltip("Load segmentation 2");
    	         	     	          Fl_Pixmap *p = new Fl_Pixmap(fileopen_xpm);
    	         	     	          p->label(o2);
    	         	     	        }
    	         	         	     }
    	         	     	        o2->end();*/


    //o->end(); // End of SegmentBrowserWindow controls

    // Create SegmentQuickEdit controls
  /*  o = new Fl_Group(0, 350, 400, 150, "SegmentQuickEdit");
    o->user_data((void*)(this));
    o->labeltype(FL_NO_LABEL);
    o->box(FL_ENGRAVED_BOX);
    {
      {
        Fl_Input *o = editSegmentLabel = new Fl_Input(10, 290, 5, 5);
        o->align(133);
        o->callback((Fl_Callback*)cb_editSegmentLabel);
      }
      {
        Fl_Slider *o = editSegmentTransperancy = new Fl_Value_Slider(10, 380, 370, 20, "Transperancy");
        o->minimum(0);
        o->maximum(1);
        o->box(FL_EMBOSSED_BOX);
        o->type(5);
        o->callback((Fl_Callback*)cb_editSegmentTransperancy);
        o->align(FL_ALIGN_TOP_LEFT);
      }
      {
        Fl_Button *o = editSegmentColor = new Fl_Button(110, 380, 10, 10, "");
        o->callback((Fl_Callback *)cb_editSegmentPickColor);
        o->type(FL_NORMAL_BUTTON);
        o->box(FL_BORDER_BOX);
        //o->label("Color chooser ");
        o->align(FL_ALIGN_LEFT);
        o->color(FL_WHITE);
      }
      {
        Fl_Check_Button *o = editSegmentVisibility = new Fl_Check_Button(275, 380, 10, 10,"");
        o->callback((Fl_Callback *)cb_editSegmentVisibility);
        o->align(FL_ALIGN_WRAP);
      }
    }
    o->end(); // End of SegmentQuickEdit controls

    // Creates histgram tools controls
    _histogramWindow = NULL;



  */}
}



#endif
